Attribute VB_Name = "DatabaseBuildingRoutines"
Attribute VB_Description = "Database building proceedures"
Option Explicit
Private Database_creation_failure As Boolean

'Declarations for the databases and tables
'NB - External keys declared under home table
'   - Tables with fields similar to those in other tables will not
'     have the fields redeclared

'Defining the Students' tables
'Students(Student ID, Surname, First Names, Class)
'Deregistered Students(Student ID, Surname, First Names, Date Deregistered)
Private tblStudents As TableDef
Private tblDeregisteredStudents As TableDef
Private fldStudentID As Field
Private fldSurname As Field
Private fldFirstNames As Field
Private fldSex As Field
Private fldDateDeregistered As Field
Private fldGuardianTitle As Field
Private fldGuardianSurname As Field
Private fldGuardianInitials As Field
Private fldGuardianAddress As Field
Private fldGuardianTelephone As Field
Private indxStudentID As Index
Private indxDateDeregistered As Index

'Classes table definitions
'Classes (Class, Form Teacher, Minimum Caution Fee Deposit)
Private tblClasses As TableDef
Private fldClass As Field
Private fldFormTeacher As Field
Private fldMinBalance As Field
Private indxClasses As Index


'Defining the Registered Fees table
'Registered Fees(Fee code, Fee Name)
Private tblRegisteredFees As TableDef
Private fldFeeCode As Field
Private fldFeeName As Field
Private indxFeeCode As Index
Private indxFeeName As Index

'Defining the Levy Account Debits table
'Levy Account Debits(Transaction Date, Amount, Fee Code, Student ID)
Private tblLevyDebits As TableDef
Private fldTransactionDate As Field
Private fldAmount As Field
Private indxTransactionDate As Index

'Defining Levy Account Credits table
'Levy Account Credits(Transaction Date, Amount, Receipt Number, Student ID)
Private tblLevyCredits As TableDef
Private fldReceiptNumber As Field
Private indxReceiptNumber As Index

'Defining Fees Templates table
'Fees Templates (Template Name, Template Code)
Private tblFeesTemplates As TableDef
Private fldTemplateName As Field
Private fldTemplateData  As Field
Private indxTemplateName As Index

'Defining Caution Fee Account Debits table
'Caution Fee Account Debits(Transaction Date, Reason, Amount, Reference Code, Student ID)
Private tblCautionDepositDebits As TableDef
Private fldReason As Field
Private fldRefCode As Field

'Defining Caution Fee Account Credits table
'Caution Fee Account Credits(Transaction Date, Amount, Receipt Number, Student ID)
Private tblCautionDepositCredits As TableDef

'Defining Users table
'Users (User name, User password)
Private tblUsers As TableDef
Private fldUserName As Field
Private fldUserPassword As Field
Private fldUserRights As Field
Private indxUserName As Index



Public Function DatabaseSuccessfullyCreated(strDBpath As String) As Boolean
    On Local Error GoTo errorHandler
    
    Dim strMsg As String
    Dim objFileSystem As New FileSystemObject
    
    Set wspLevy = DBEngine.Workspaces(0)
    
    'Search for databses and if it exists abort creation of database
    If Dir(strDBpath) <> "" Then
        strMsg = "Unable to create database because it already exits."
        strMsg = strMsg + strNewLine + "To create a new database delete the old one first."
        MsgBox strMsg, vbCritical
        DatabaseSuccessfullyCreated = False
    Else
        'Create a blank database
        Set dbLevyOffice = wspLevy.CreateDatabase(strDBpath, dbLangGeneral & ";pwd=25081982", dbEncrypt)
         
        'Create Tables
        Create_Students_table
        Create_Deregistered_Students_table
        Create_Classes_table
        Create_Registered_Fees_table
        Create_Fees_Templates_table
        Create_Levy_Account_Debits_table
        Create_Levy_Account_Credits_table
        Create_Caution_Fee_Account_Debits_table
        Create_Caution_Fee_Account_Credits_table
        Create_users_table
        
        'Create query objects
        Set qryQueryObject = dbLevyOffice.CreateQueryDef("General Query Obj")
        Set qryReports = dbLevyOffice.CreateQueryDef("Report Query Object")
        Set qryDirectSQL = dbLevyOffice.CreateQueryDef("Direct SQL Query Object")
        DatabaseSuccessfullyCreated = True
    End If
    
Exit Function

errorHandler:
    MsgBox Err.Description, vbExclamation
    DatabaseSuccessfullyCreated = False
        

End Function

Public Sub Create_Students_table()
    
    'Students(Student ID, Surname, First Names, Class)
    Set tblStudents = dbLevyOffice.CreateTableDef("Students")
       
    Set fldStudentID = tblStudents.CreateField("Student ID", dbText, 8)
    Set fldSurname = tblStudents.CreateField("Surname", dbText, 20)
    Set fldFirstNames = tblStudents.CreateField("First Names", dbText, 30)
    Set fldClass = tblStudents.CreateField("Class", dbText, 6)
    Set fldSex = tblStudents.CreateField("Sex", dbText, 1)
    Set fldGuardianTitle = tblStudents.CreateField("Guardian Title", dbText, 6)
    Set fldGuardianSurname = tblStudents.CreateField("Guardian Surname", dbText, 20)
    Set fldGuardianInitials = tblStudents.CreateField("Guardian Initials", dbText, 6)
    Set fldGuardianAddress = tblStudents.CreateField("Guardian Address", dbText, 150)
    Set fldGuardianTelephone = tblStudents.CreateField("Guardian Telephone", dbText, 15)

    
    'Write the fields to the table
    tblStudents.Fields.Append fldStudentID
    tblStudents.Fields.Append fldSurname
    tblStudents.Fields.Append fldFirstNames
    tblStudents.Fields.Append fldSex
    tblStudents.Fields.Append fldClass
    tblStudents.Fields.Append fldGuardianTitle
    tblStudents.Fields.Append fldGuardianSurname
    tblStudents.Fields.Append fldGuardianInitials
    tblStudents.Fields.Append fldGuardianAddress
    tblStudents.Fields.Append fldGuardianTelephone

    
    'Write table to database
    dbLevyOffice.TableDefs.Append tblStudents

    'Sorting index for Student ID
    Set indxStudentID = tblStudents.CreateIndex("Student ID Index")
    Set fldStudentID = indxStudentID.CreateField("Student ID")
    indxStudentID.Fields.Append fldStudentID
    tblStudents.Indexes.Append indxStudentID
    
    
End Sub

Public Sub Create_Deregistered_Students_table()
    
    'Deregistered Students(Student ID, Surname, First Names, Date Deregistered)
    Set tblDeregisteredStudents = dbLevyOffice.CreateTableDef("Deregistered Students")
       
    Set fldStudentID = tblDeregisteredStudents.CreateField("Student ID", dbText, 8)
    Set fldSurname = tblDeregisteredStudents.CreateField("Surname", dbText, 20)
    Set fldFirstNames = tblDeregisteredStudents.CreateField("First Names", dbText, 30)
    Set fldSex = tblDeregisteredStudents.CreateField("Sex", dbText, 1)
    Set fldDateDeregistered = tblDeregisteredStudents.CreateField("Date Deregistered", dbDate)
    Set fldGuardianTitle = tblDeregisteredStudents.CreateField("Guardian Title", dbText, 6)
    Set fldGuardianSurname = tblDeregisteredStudents.CreateField("Guardian Surname", dbText, 20)
    Set fldGuardianInitials = tblDeregisteredStudents.CreateField("Guardian Initials", dbText, 6)
    Set fldGuardianAddress = tblDeregisteredStudents.CreateField("Guardian Address", dbText, 150)
    Set fldGuardianTelephone = tblDeregisteredStudents.CreateField("Guardian Telephone", dbText, 15)

    'Write the fields to the table
    tblDeregisteredStudents.Fields.Append fldStudentID
    tblDeregisteredStudents.Fields.Append fldSurname
    tblDeregisteredStudents.Fields.Append fldFirstNames
    tblDeregisteredStudents.Fields.Append fldSex
    tblDeregisteredStudents.Fields.Append fldDateDeregistered
    tblDeregisteredStudents.Fields.Append fldGuardianTitle
    tblDeregisteredStudents.Fields.Append fldGuardianSurname
    tblDeregisteredStudents.Fields.Append fldGuardianInitials
    tblDeregisteredStudents.Fields.Append fldGuardianAddress
    tblDeregisteredStudents.Fields.Append fldGuardianTelephone
    
    'Write table to database
    dbLevyOffice.TableDefs.Append tblDeregisteredStudents

    'Sorting index for Stuent ID
    Set indxStudentID = tblDeregisteredStudents.CreateIndex("Student ID Index")
    Set fldStudentID = indxStudentID.CreateField("Student ID")
    indxStudentID.Fields.Append fldStudentID
    tblDeregisteredStudents.Indexes.Append indxStudentID

End Sub

Public Sub Create_Classes_table()
    ' Classes (Class, Form Teacher, Minimum Caution Fee Deposit)

    Set tblClasses = dbLevyOffice.CreateTableDef("Classes")
       
    Set fldClass = tblClasses.CreateField("Class", dbText, 6)
    Set fldFormTeacher = tblClasses.CreateField("Form Teacher", dbText, 50)
    Set fldMinBalance = tblClasses.CreateField("Minimum Balance", dbCurrency)
    
    'Write the fields to the table
    tblClasses.Fields.Append fldClass
    tblClasses.Fields.Append fldFormTeacher
    tblClasses.Fields.Append fldMinBalance
    
    'Write table to database
    dbLevyOffice.TableDefs.Append tblClasses

    'Sorting index for Class
    Set indxClasses = tblClasses.CreateIndex("Class Index")
    Set fldClass = indxStudentID.CreateField("Class")
    indxClasses.Fields.Append fldClass
    tblClasses.Indexes.Append indxClasses

    
End Sub

Public Sub Create_Registered_Fees_table()
    
    ' Registered Fees (Fee code, Fee Name)
    Set tblRegisteredFees = dbLevyOffice.CreateTableDef("Registered Fees")
       
    Set fldFeeCode = tblRegisteredFees.CreateField("Fee Code", dbInteger)
    Set fldFeeName = tblRegisteredFees.CreateField("Fee Name", dbText, 30)
        
    'Write the fields to the table
    tblRegisteredFees.Fields.Append fldFeeCode
    tblRegisteredFees.Fields.Append fldFeeName
    
    
    'Write table to database
    dbLevyOffice.TableDefs.Append tblRegisteredFees

    'Sorting index for Fee Code
    Set indxFeeCode = tblRegisteredFees.CreateIndex("Fee Code Index")
    Set fldFeeCode = indxFeeCode.CreateField("Fee Code")
    indxFeeCode.Fields.Append fldFeeCode
    tblRegisteredFees.Indexes.Append indxFeeCode
    
    'Sorting Index for Fee Name
    Set indxFeeName = tblRegisteredFees.CreateIndex("Fee Name Index")
    Set fldFeeName = indxFeeName.CreateField("Fee Name")
    indxFeeName.Fields.Append fldFeeName
    tblRegisteredFees.Indexes.Append indxFeeName
    
  
End Sub

Public Sub Create_Fees_Templates_table()
    'Fees Templates(Template Name, Template Code)

    Set tblFeesTemplates = dbLevyOffice.CreateTableDef("Fees Templates")
    
    Set fldTemplateName = tblFeesTemplates.CreateField("Template Name", dbText, 30)
    Set fldTemplateData = tblFeesTemplates.CreateField("Template Data", dbMemo)
    fldTemplateData.AllowZeroLength = True
    
    'Write the fields to the table
    tblFeesTemplates.Fields.Append fldTemplateName
    tblFeesTemplates.Fields.Append fldTemplateData
    
    
    'Write table to database
    dbLevyOffice.TableDefs.Append tblFeesTemplates

    'Sorting index for Template Name
    Set indxTemplateName = tblFeesTemplates.CreateIndex("Template Name Index")
    Set fldTemplateName = indxTemplateName.CreateField("Template Name")
    indxTemplateName.Fields.Append fldTemplateName
    tblFeesTemplates.Indexes.Append indxTemplateName
    
End Sub

Public Sub Create_Levy_Account_Debits_table()
    'Levy Account Debits (Transaction Date, Amount, Fee Code, Student ID)

    Set tblLevyDebits = dbLevyOffice.CreateTableDef("Levy Account Debits")
       
    Set fldTransactionDate = tblLevyDebits.CreateField("Transaction Date", dbDate)
    Set fldAmount = tblLevyDebits.CreateField("Amount", dbCurrency)
    Set fldFeeCode = tblLevyDebits.CreateField("Fee Code", dbInteger)
    Set fldStudentID = tblLevyDebits.CreateField("Student ID", dbText, 8)
    
    'Write the fields to the table
    tblLevyDebits.Fields.Append fldTransactionDate
    tblLevyDebits.Fields.Append fldAmount
    tblLevyDebits.Fields.Append fldFeeCode
    tblLevyDebits.Fields.Append fldStudentID
    
    'Write table to database
    dbLevyOffice.TableDefs.Append tblLevyDebits
   
End Sub

Public Sub Create_Levy_Account_Credits_table()
    'Levy Account Credits (Transaction Date, Amount, Receipt Number, Student ID)

    Set tblLevyCredits = dbLevyOffice.CreateTableDef("Levy Account Credits")
       
    Set fldTransactionDate = tblLevyCredits.CreateField("Transaction Date", dbDate)
    Set fldAmount = tblLevyCredits.CreateField("Amount", dbCurrency)
    Set fldReceiptNumber = tblLevyCredits.CreateField("Receipt Number", dbText, 15)
    Set fldStudentID = tblLevyCredits.CreateField("Student ID", dbText, 8)
    
    'Write the fields to the table
    tblLevyCredits.Fields.Append fldTransactionDate
    tblLevyCredits.Fields.Append fldAmount
    tblLevyCredits.Fields.Append fldReceiptNumber
    tblLevyCredits.Fields.Append fldStudentID
    
    'Receipt Number Index
    Set indxReceiptNumber = tblLevyCredits.CreateIndex("Receipt Number Index")
    Set fldReceiptNumber = indxReceiptNumber.CreateField("Receipt Number")
    indxReceiptNumber.Fields.Append fldReceiptNumber
    tblLevyCredits.Indexes.Append indxReceiptNumber
    
    'Write table to database
    dbLevyOffice.TableDefs.Append tblLevyCredits
   
End Sub


Public Sub Create_Caution_Fee_Account_Debits_table()
    'Caution Fee Account Debits(Transaction Date, Reason, Amount, Reference Code, Student ID)
    Set tblCautionDepositDebits = dbLevyOffice.CreateTableDef("Caution Fee Account Debits")
       
    Set fldTransactionDate = tblCautionDepositDebits.CreateField("Transaction Date", dbDate)
    Set fldReason = tblCautionDepositDebits.CreateField("Reason", dbText, 30)
    Set fldAmount = tblCautionDepositDebits.CreateField("Amount", dbCurrency)
    Set fldRefCode = tblCautionDepositDebits.CreateField("Reference code", dbText, 15)
    Set fldStudentID = tblCautionDepositDebits.CreateField("Student ID", dbText, 8)
    
    'Write the fields to the table
    tblCautionDepositDebits.Fields.Append fldTransactionDate
    tblCautionDepositDebits.Fields.Append fldReason
    tblCautionDepositDebits.Fields.Append fldAmount
    tblCautionDepositDebits.Fields.Append fldRefCode
    tblCautionDepositDebits.Fields.Append fldStudentID
    
    'Write table to database
    dbLevyOffice.TableDefs.Append tblCautionDepositDebits
   
End Sub

Public Sub Create_Caution_Fee_Account_Credits_table()
    'Caution Fee Account Credits(Transaction Date, Amount, Receipt Number, Student ID)

    Set tblCautionDepositCredits = dbLevyOffice.CreateTableDef("Caution Fee Account Credits")
       
    Set fldTransactionDate = tblCautionDepositCredits.CreateField("Transaction Date", dbDate)
    Set fldAmount = tblCautionDepositCredits.CreateField("Amount", dbCurrency)
    Set fldReceiptNumber = tblCautionDepositCredits.CreateField("Receipt Number", dbText, 15)
    Set fldStudentID = tblCautionDepositCredits.CreateField("Student ID", dbText, 8)
    
    'Write the fields to the table
    tblCautionDepositCredits.Fields.Append fldTransactionDate
    tblCautionDepositCredits.Fields.Append fldAmount
    tblCautionDepositCredits.Fields.Append fldReceiptNumber
    tblCautionDepositCredits.Fields.Append fldStudentID
    
    'Receipt Number Index
    Set indxReceiptNumber = tblCautionDepositCredits.CreateIndex("Receipt Number Index")
    Set fldReceiptNumber = indxReceiptNumber.CreateField("Receipt Number")
    indxReceiptNumber.Fields.Append fldReceiptNumber
    tblCautionDepositCredits.Indexes.Append indxReceiptNumber
    
    'Write table to database
    dbLevyOffice.TableDefs.Append tblCautionDepositCredits
   
End Sub


Public Sub Create_users_table()
    
    Set tblUsers = dbLevyOffice.CreateTableDef("Users")
    Set fldUserName = tblUsers.CreateField("User Name", dbText, 30)
    Set fldUserPassword = tblUsers.CreateField("User Password", dbText, 32)
    Set fldUserRights = tblUsers.CreateField("User Rights", dbText, 30)
        
    
    tblUsers.Fields.Append fldUserName
    tblUsers.Fields.Append fldUserPassword
    tblUsers.Fields.Append fldUserRights
    
    dbLevyOffice.TableDefs.Append tblUsers
    
    'Sorting index for user name
    Set indxUserName = tblUsers.CreateIndex("User Name Index")
    Set fldUserName = indxUserName.CreateField("User Name")
    indxUserName.Fields.Append fldUserName
    tblUsers.Indexes.Append indxUserName
    
    
    
End Sub



